﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MathNet.Numerics.Interpolation.Algorithms;
using Statistics;


namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        NevillePolynomialInterpolation interpolation;

        public Form1()
        {
            InitializeComponent();
        }

        private static double GetPolynomResult(double[] lagrange, double x)
        {
            var result = lagrange[0];
            for (var j = 1; j < lagrange.Count(); j++)
                result += lagrange[j] * Math.Pow(x, j);
            return result;
        }
        
        private static void CalcApproximateValue(
            double[] xsrc, double[] y, int degree, out double[] lagrange)
        {
            var count = y.Count();
            var x = new double[degree + 1, count];
            var w = new double[count];
            for (var i = 0; i < count; i++)
            {
                x[0, i] = 1;
                double xx = xsrc[i];
                var term = xx;
                for (var j = 1; j <= degree; j++)
                {
                    x[j, i] = term;
                    term *= xx;
                }
                w[i] = 1;
            }
            var regression = new LinearRegression();
            regression.Regress(y, x, w);
            lagrange = new double[regression.C.Count()];
            regression.C.CopyTo(lagrange, 0);
        }
        
        private void button1_Click(object sender, EventArgs e)
        {
            int firstend = (int)numericUpDown1.Value;
            int addx = (int)numericUpDown2.Value;
            double[] x;
            double[] y;
            double[] lagrange = null;
            FillXY(firstend, out x, out y);
            CalcApproximateValue(x, y, (int)numericUpDown3.Value, out lagrange);

            var interpolation = new NevillePolynomialInterpolation();
            interpolation.Initialize((IList<double>)x, (IList<double>)y);

            FillXY(firstend + addx, out x, out y);

            chart1.Series[0].Points.Clear();
            chart1.Series[1].Points.Clear();
            chart1.Series[2].Points.Clear();

            for (var i = 0; i < firstend + addx; i++)
            {
                if (i < firstend)
                    chart1.Series[0].Points.AddXY(x[i], y[i]);
                if (checkBox1.Checked)
                    chart1.Series[1].Points.AddXY(x[i], interpolation.Interpolate(x[i]));
                chart1.Series[2].Points.AddXY(x[i], GetPolynomResult(lagrange, (double)i / 3));
            }
            chart1.Refresh();
        }

        private static void FillXY(int count, out double[] x, out double[] y)
        {
            x = new double[count];
            y = new double[count];
            for (var i = 0; i < count; i++)
            {
                x[i] = (double)i / 3;
                y[i] = Math.Sin(x[i]);
            }
        }

        private void button1_Click_1(object sender, EventArgs e)
        {
            webBrowser1.Visible = true;
            webBrowser1.Navigate(new Uri("http://nikitayev.livejournal.com"));
        }

        
    }
}
